/*
By J.Shuhy
www.Shuhy.com

The MIT License (MIT)

Copyright (c) 2014 J.Shuhy

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.


Arduino       Bluetooth module
5V            PIN 2 (VCC),
GND           PIN 3 (GND)
DIO 10        PIN 4 (TXD)
DIO 11        PIN 5 (RXD)


100PSI fuel sensor
Red +5V
Black ground
Blue signal 0.5V - 4.5V is linear 0 - 100 PSI signal


3 bar MAP
black ground
yellow signal 0 - 5V is linear -15 to 30 PSI
white +5V

Wideband
0.15 to 3.27 V, non-linear

V       AFR
0.15	9.7
0.31	9.9
0.46	10.1
0.62	10.3
0.78	10.5
0.93	10.7
1.09	11
1.24	11.4
1.4	11.7
1.56	12.1
1.71	12.4
1.87	12.8
2.02	13.2
2.18	13.7
2.34	14.2
2.5	14.7
2.65	15.6
2.8	16.9
2.96	18.5
3	18.8
3.12	19.9
3.27	21.2

LM358N
1 output-1
2 input-1 -
3 input-1 +
4 gnd
5 input-2 +
6 input-2 -
7 output-2
8 Vcc

GY-61 Accelerometer ADXL335 
Vcc +3.3V
X out
Y out
Z out
Gnd


Arduino Analog inputs
A0 - 0 to 100PSI fuel
A1 - -15 to 30PSI Boost
A2 - Spare
A3 - WB O2
A4 - x axis
A5 - y axis
A6 - z axis


Connector
1 Fuel 
2 Boost
3 5V In 
4 WB In(USB power is 4.x volts and creates bad sensor readings)
5 WB Out
6 Chassis Ground

Car:
Yellow wire goes to MAP sensor
Green wire goes to fuel PSI
12V to 5V regulator connects to gauges fuse
Red wire feeds 5V to sensors
Ground from regulator goes to sensors and chassis

*/

/* Include the software serial port library */
#include <SoftwareSerial.h>

/* DIO used to communicate with the Bluetooth module's TXD pin */
#define BT_SERIAL_TX_DIO 10 
/* DIO used to communicate with the Bluetooth module's RXD pin */
#define BT_SERIAL_RX_DIO 11

#define FP_INPUT      A0
#define BOOST_INPUT   A1
#define SPARE_INPUT   A2
#define AFR_INPUT     A3
#define Y_INPUT       A4
#define X_INPUT       A5
// A6 is not working properly, read returns value from last input read
//#define Z_INPUT       A6

#define MAX_INPUT    1023.0

// manual calibration is needed - rotate acceleromter in all directions and note max & min values obtained for each axis
// then put those values here, they will define a 2G range for each axis (-1G to +1G)

#define MAX_X   425
#define MIN_X   279
#define MAX_Y   425
#define MIN_Y   276
//#define MAX_Z   410
//#define MIN_Z   252

/* Initialise the software serial port */
SoftwareSerial BluetoothSerial(BT_SERIAL_TX_DIO, BT_SERIAL_RX_DIO);

double gXCal = (MAX_X + MIN_X) / 2;
double gYCal = (MAX_Y + MIN_Y) / 2;
//double gZCal = (MAX_Z + MIN_Z) / 2;;

void accelCal()
{
  int i;
  int loops = 50;
  
  gXCal = 0;
  gYCal = 0;
//  gZCal = 0;
  
  for (i = 0; i < loops; i++)
  {
     gXCal += analogRead(X_INPUT);
     gYCal += analogRead(Y_INPUT);
//     gZCal += analogRead(Z_INPUT);
     delay(20);
  }
  
  gXCal /= loops;
  gYCal /= loops;
//  gZCal /= loops;
}

double calcAFR(double reading)
{
    #define AFR_PAIRS 24
    double afrVoltages[AFR_PAIRS] = {0.0, 0.15, 0.31, 0.46, 0.62, 0.78, 0.93, 1.09, 1.24, 1.4, 1.56, 1.71, 1.87, 
                                    2.02, 2.18, 2.34, 2.5, 2.65, 2.8, 2.96, 3, 3.12, 3.27, 5.0};
    double afrValues[AFR_PAIRS] = {9.0, 9.7, 9.9, 10.1, 10.3, 10.5, 10.7, 11, 11.4, 11.7, 12.1, 12.4, 12.8, 
                                 13.2, 13.7, 14.2, 14.7, 15.6, 16.9, 18.5, 18.8, 19.9, 21.2, 25.0};

    int i = 0;
    double voltage = (reading/MAX_INPUT) * 5.0;
        
    while (i < AFR_PAIRS && voltage >= afrVoltages[i]) i++;

    if (i >= AFR_PAIRS)
    {
      return afrValues[AFR_PAIRS-1];
    }
    else if (i == 0)
    {
      // shouldn't get here
      return afrValues[0];
    }

    return ((afrValues[i] - afrValues[i-1]) / (afrVoltages[i] - afrVoltages[i-1])) * (voltage - afrVoltages[i-1]) +  afrValues[i-1];
}

int avgReading(int input, int numReads)
{
  int i;
  int reading = 0;
  
  if (numReads == 0) return 0;
  
  for (i=0; i<numReads; i++)
  {
    reading += analogRead(input);
  }
  
/*
  Serial.print("input: ");
  Serial.print(input);
  Serial.print(", total: ");
  Serial.print(reading);
  Serial.print(", avg: ");
  Serial.println(reading/numReads);
*/

  return (reading / numReads); 
}


void setup()  
{
  Serial.begin(115200);
  Serial.println("Gauges v0.66");
    
  /* Set the baud rate for the software serial port */
  BluetoothSerial.begin(9600);
  BluetoothSerial.print("AT+NAMErPi");
  
  //maxMinCal();
  
  accelCal();
}

void loop()
{
  double afrReading;
  int fpReading;
  double boostReading;
  double xReading;
  double yReading;
//  double zReading;
  int i;

  static char button[3];
  static int buttonChars = 0;
  static char command[3];
  static int commandChars = 0;

  // fuel pressire 0.5V = 0 PSI; 4.5V = 100 PSI
  fpReading = ((avgReading(FP_INPUT, 10)/MAX_INPUT) * 125) - 12.5;
  fpReading = max(fpReading,0);
  i = avgReading(FP_INPUT, 10);
/*
2.35v 480 raw
  Serial.print("fp avg: ");
  Serial.print((i * 5.0) / MAX_INPUT);
  Serial.print(", raw: ");
  Serial.println(analogRead(FP_INPUT));
*/
  
  // boost 0V = -15 PSI; 5V = 30 PSI
  boostReading = ((avgReading(BOOST_INPUT, 10)/MAX_INPUT) * 45) - 15;
  i = avgReading(BOOST_INPUT, 10);
/*
1.74v 356 raw
  Serial.print("map avg: ");
  Serial.print((i * 5.0)/ MAX_INPUT);
  Serial.print(", raw: ");
  Serial.println(analogRead(BOOST_INPUT));
*/
  // afr is non-linear
  afrReading = calcAFR(avgReading(AFR_INPUT, 10));
  
  // accel axis  
  xReading = (((avgReading(X_INPUT,20)) - gXCal) * 2.0) / (MAX_X - MIN_X);
  yReading = (((avgReading(Y_INPUT,20)) - gYCal) * 2.0) / (MAX_Y - MIN_Y);
  //zReading = (((avgReading(Z_INPUT,20)) - gZCal) * 2.0) / (MAX_Z - MIN_Z);
  
  Serial.print("a");
  Serial.println(afrReading);

  Serial.print("f");
  Serial.println(fpReading);

  Serial.print("p");
  Serial.println(boostReading);

  Serial.print("x");
  Serial.println(xReading);
  Serial.print("y");
  Serial.println(yReading);
 
  // bluetooth is used for remote control
  // If data is available from the Bluetooth module then pass it on to the hardware serial port.
  while (BluetoothSerial.available())
  {
    // 2 letter button commands "b_"
    button[buttonChars] = BluetoothSerial.read();
    buttonChars++;
    
    if (button[0] != 'b')
    {
        buttonChars = 0;
    }

    if (buttonChars >= 2)
    { 
       button[2] = 0;    
       Serial.println(button);
       buttonChars = 0;
    }
  }
 
  while (Serial.available())
  {
    // 2 letter commands from rPi "c_"
    command[commandChars] = Serial.read();
    commandChars++;
    
    if (command[0] != 'c')
    {
        commandChars = 0;
    }

    if (commandChars >= 2)
    { 
       command[2] = 0;    
       // process command
       if (command[1] == 'c')
       {
         // calibrate command
         accelCal();
       }
       else
       {
         // unknow command
       }
       
       buttonChars = 0;
    }    
  }
  
  delay(200);
}


void maxMinCal()
{
  int xRaw, xMax=0, xMin=1023;
  int yRaw, yMax=0, yMin=1023;
//  int zRaw, zMax=0, zMin=1023;
  
  int i;
  int raw, j, loops=20;
  
  Serial.println("Max Min Loop");
  
  for (i=0; i<100000; i++)
  {
    raw = 0;
    for (j=0; j<loops; j++)
    {
      raw += analogRead(X_INPUT);
    }
    xRaw = raw / loops;
    
    raw = 0;
    for (j=0; j<loops; j++)
    {
      raw += analogRead(Y_INPUT);
    }
    yRaw = raw / loops;

//    zRaw = analogRead(Z_INPUT);
    
    if (xRaw < xMin)
    {
      Serial.print("X:");
      Serial.println(xRaw);      
      xMin = xRaw;
    }
    if (xRaw > xMax)
    {
      Serial.print("X:");
      Serial.println(xRaw);      
      xMax = xRaw;
    }
    
    if (yRaw < yMin)
    {
      Serial.print("Y:");
      Serial.println(yRaw);      
      yMin = yRaw;
    }
    if (yRaw > yMax)
    {
      Serial.print("Y:");
      Serial.println(yRaw);      
      yMax = yRaw;
    }
  
/*
    if (zRaw < zMin)
    {
      zMin = zRaw;
    }
    if (zRaw > zMax)
    {
      zMax = zRaw;
    }
*/
    
    if (i%500 == 0)
    {
      Serial.print("X:");
      Serial.print(xMin);
      Serial.print(",");
      Serial.print(xMax);
      Serial.print(",");
      Serial.print(xRaw);
      
      Serial.print("; Y:");
      Serial.print(yMin);
      Serial.print(",");
      Serial.print(yMax);
      Serial.print(",");
      Serial.println(yRaw);

/*
      Serial.print("; Z:");
      Serial.print(zMin);
      Serial.print(",");
      Serial.print(zMax);
      Serial.print(",");
      Serial.println(zRaw);
*/
    }
  }
}
